% Purpose:  Generating a signal to a flap wave maker utilizing the JONSWAP
%           and/or Donelan wave spectrum and transfer function for the 
%           flap. Reads in an .txt-file and generates sea states as 
%           described.
%           NB!! Flap motion must not overgo 0.3 m.
% ------------------------------------------------------------------------
% Variable                 I/O      Description
% ------------------------------------------------------------------------
% wmsignal                  O        Signal sent to wavemaker
% trigsignal                O        Start/stop signal for wavemeters
% 'testinfo_irregular.txt'  I        Inputfile 
% ------------------------------------------------------------------------
% Calling functions:    jonswap(w,Hs,Tp,gamma)
%                       kiteration(w,d)
%                       piston_trans(k,d)
%                       write2mcl(['mcl/' savename],Data)
% ------------------------------------------------------------------------
% January 2016
% Overn, Ida Haftun 
% Waage, Ida Fredsvik
% ------------------------------------------------------------------------

clear
format compact
warning off

tic
Fs    = 50;     %sampling frequency of wavemaker
dt    = 1/Fs;   %corresponding time-interval
scale = 100;    %modelscale 
gg    = 9.81;   %gravity 

filename       = 'testinfo_irregular.txt';   %file describing the tests
testinfo       = load(filename);

kk = 1;
idvec          = testinfo(:,kk); kk = kk+1;
Tpvec          = testinfo(:,kk); kk = kk+1;
Hsminvec       = testinfo(:,kk); kk = kk+1;
Hsmaxvec       = testinfo(:,kk); kk = kk+1;
deltaHsvec     = testinfo(:,kk); kk = kk+1;
gammavec       = testinfo(:,kk); kk = kk+1;
wdepthvec      = testinfo(:,kk); kk = kk+1;
statelengthvec = testinfo(:,kk); kk = kk+1;
waitingtimevec = testinfo(:,kk); kk = kk+1;
nrepeatsvec    = testinfo(:,kk); kk = kk+1;
Ntests         = length(idvec);

Nt = ceil(16/dt);
wmsignal = zeros(1,16/dt+1);
trigsignal = zeros(1,16/dt+1);
tvec = [0:dt:16];

n_Hs = (Hsmaxvec(1)-Hsminvec(1))/deltaHsvec(1) + 1;

for itest = 1:Ntests           %Tp-values 
    Tp = Tpvec(itest);
    Tp_m = Tp/sqrt(scale);
    d_m = wdepthvec(itest)/scale;
    N = round(statelengthvec(itest)*60*60/sqrt(scale));  
    dw = 2*pi/N;
    w = [dw:dw:30];
    nlines_state = ceil(N/dt);
    nrepeat = nrepeatsvec(itest);
    for Hs = Hsminvec(itest):deltaHsvec(itest):Hsmaxvec(itest) %Hs values 
        Hs_m = Hs/scale;
        s2 = donelan(w,Hs_m,Tp_m);
        %s2 = jonswap(w,Hs_m,Tp_m,gammavec(itest));
        k_m = kiteration(w,d_m);
        h12 = biesel_flap(k_m,d_m,0.08);
        s1 = s2./(abs(h12).^2);
        theta = zeros(1,length(w));
        random_number = zeros(1,length(w));
        for rep = 1:nrepeatsvec(itest)      %number of repeats 
            for i = 1:length(w)
                theta(i) = rand*2*pi;       %random phase angle
                random_number(i) = rand;    %Rayleigh probability value
            end
            s3 = sqrt(-dw*2*s2.*log(1-random_number)./h12);
            amp = sqrt(-dw*2*s2.*log(1-random_number));
            trigsig_active = ones(1,N/dt+1);
            trigsignal = [trigsignal, trigsig_active];
            wave = zeros(1,round(N/dt));
            l = 0;
            for t = 0:dt:N
                Nt = Nt+1;
                l = l + 1;
                tvec = [tvec, tvec(end)+dt];
                wmsignal_t = sum(s3.*cos(w*t-theta))*1000;
                wmsignal = [wmsignal,wmsignal_t];
            end
            nlines_wait = round(waitingtimevec(itest)/dt);
            Nt_w = Nt + nlines_wait;
            wmsignal = [wmsignal,zeros(1,nlines_wait)];
            trigsignal = [trigsignal, zeros(1,nlines_wait)];
            tvec_wait = [tvec(end)+dt:dt:tvec(end)+...
                round(waitingtimevec(itest))];
            tvec = [tvec, tvec_wait];
        end
    end
end

wmsignal = (wmsignal./(wdepthvec(1)/scale-0.08)).*(wdepthvec(1)/scale-0.08+0.418);

trigsignal(end-10:end) = 0;
nlines_tot = length(wmsignal);
plot(tvec,wmsignal)
plot(tvec/3600,wmsignal/1000,'b')
grid on, hold on
plot(tvec/3600,trigsignal*max(wmsignal/1000),'g')
xlabel('Time (hours)')
ylabel('Wm motion (m)')
charid  = num2str(idvec(1));
savename  = [charid '.dat'];

Data      = zeros(nlines_tot,6);
Data(:,1) = wmsignal;
Data(:,2) = trigsignal;
write2mcl(['mcl/' savename],Data);
            
toc