%% Matlab script "generateRegWaves.m"
%
% Purpose:
%      Generate regular time-series to
%      wave flap in Lilletanken
%
%
% 23.10.2015,  Trygve Kristiansen, NTNU  (Master students)
%
%%

clear
format compact
warning off

%% Parameters:
Fs    = 50;     % Sampling frequency of wavemaker
dt     = 1/Fs;

scale = 100;
gg    = 9.81;
DD    = 6.9;      % Diameter of monopile
Lm    = 15;       % Length from wm to monopile

fsize  = 14;

%% Load testinfo:
filename       = 'testinfo.txt';            % Test info file describing the tests
testinfo       = load(filename);
kk = 1;
testvec        = testinfo(:,kk); kk = kk+1;
testwvec       = testinfo(:,kk); kk = kk+1;
testwmvec      = testinfo(:,kk); kk = kk+1;
Tminvec        = testinfo(:,kk); kk = kk+1;
Tmaxvec        = testinfo(:,kk); kk = kk+1;
deltaTvec      = testinfo(:,kk); kk = kk+1;
Steepminvec    = testinfo(:,kk); kk = kk+1;
Steepmaxvec    = testinfo(:,kk); kk = kk+1;
deltaSteepvec  = testinfo(:,kk); kk = kk+1;
wdepthvec      = testinfo(:,kk); kk = kk+1;
npervec        = testinfo(:,kk); kk = kk+1;
nrampvec       = testinfo(:,kk); kk = kk+1;
tzerovec       = testinfo(:,kk); kk = kk+1;
icorrectvec    = testinfo(:,kk); kk = kk+1;
alphavec       = testinfo(:,kk); kk = kk+1;
betavec        = testinfo(:,kk); kk = kk+1;
tauvec         = testinfo(:,kk); kk = kk+1;
Ntests         = length(testvec);

isave = 1;

%% Produce wave maker control files:
for itest = 1:Ntests
  
  %% Create initally 5s 0-valued time-series to avoid built-in ramp:
  Time      = [0:dt:5]';
  nt        = length(Time);
  Wmsig     = zeros(nt,1);
  Trigsig   = zeros(nt,1);
  Nt        = nt;
  
  %% Test id:
  testid  = testwmvec(itest);
  charid  = num2str(testid);
  disp([' ... Producing wm control file : ' charid ])
  
  hh     = wdepthvec(itest)/scale
  nramp  = nrampvec(itest);
  nper   = npervec(itest);
  tzero  = tzerovec(itest);
  alpha  = alphavec(itest);
  beta   = betavec(itest);
  tau    = tauvec(itest);
  
  %% Create Tvec and steepvec:
  Tvec     = [Tminvec(itest):deltaTvec(itest):Tmaxvec(itest)];
  steepvec = [Steepminvec(itest):-deltaSteepvec(itest):Steepmaxvec(itest)];
  Nper     = length(Tvec);
  Nsteep   = length(steepvec);
  
  %% Load mechanical transfer function:
  %MechTF
  
  kk = 1;
  for isteep = 1:Nsteep
    steep = steepvec(isteep);
    for iper = 1:Nper
      
      Tper    = Tvec(iper)/sqrt(scale);
      om      = 2*pi/Tper;
      
      %% Find amplitude of wave at given water depth:
      kkw       = findk(Tper,hh);
      lambda    = 2*pi/kkw;
      lambdavec(kk) = lambda;
      ampWave       = 0.5*steep^-1*lambda;
      AmpWave(iper) = ampWave;
      
      %% Find amplitude of flap motion at given water depth:
      %HS2  = biesel3(Tper,hh,0.08,0);
      HS3 = biesel_flap(kkw,hh,0.08);
      %TF             = interp1(TFData{1}*2*pi,TFData{2},om,'linear','extrap');
      TF = 1;
      ampFlap        = ampWave/HS3/TF;
      ampFlapvec(kk) = ampFlap;
      
      %% Create signal:
      maxFlap = 0.4; % Restraint for wm flap maximum range on rails
      if (ampFlap < maxFlap)
        
        %% Create time-signal - nper wave periods including nramp+nramp period ramps:
        nt    = round(Tper/dt*nper);
        Nt    = Nt + nt;
        time  = [1:nt]'*dt;
        wmsig = ampFlap*sin(om*time);
        
        %% Apply linear ramps at beginning and end:
        ntr   = round(Tper/dt*nramp);
        ramp  = [0:ntr-1]'/(ntr-1);
        wmsig(1:ntr)         = wmsig(1:ntr).*ramp;
        wmsig(end-ntr+1:end) =  wmsig(end-ntr+1:end).*flipud(ramp);
        
        %% Construct trigger signal (1 during wave, 0 during pause):
        npause  = round(tzero*Fs);
        [T,k,Cg,Cg_deep] = linwave2('T',Tper,'S',steep,DD,hh,0);
        ntravel = round(Lm/Cg)*Fs;
        ntrig   = min(npause,ntravel);
        trigsig = ones(nt + ntrig,1);
        
        %% Append this time-signal:
        t1    = Time(end);
        Time  = [Time  ; time + t1];
        Wmsig = [Wmsig ; wmsig];
        Trigsig = [Trigsig ; trigsig];
        
        %% Append tzero seconds of zero or small-amplitude motion:
        nt    = round(tzero*Fs);
        Nt    = Nt + nt;
        time  = [1:nt]'*dt;
        wmsig = zeros(nt,1);
        %trigsig = zeros(nt - ntrig,1);
        trigsig = ones(nt - ntrig,1);
        t1    = Time(end);
        Time  = [Time ; time+t1];
        Wmsig = [Wmsig ; wmsig];
        Trigsig = [Trigsig ; trigsig];
        
        kk = kk+1;
      end
    end
  end
  Trigsig(end-10:end) = 0;
  
  Wmsig = (Wmsig/(hh-0.08))*(hh-0.08+0.4);
  
  %% Plot:
  clf
  set(gca,'FontSize',fsize)
  savename  = [charid '.dat'];
  
  ivec = [1:1:Nt];
  plot(Time(ivec)/3600,Wmsig(ivec),'b')
  grid on, hold on
  plot(Time(ivec)/3600,Trigsig(ivec)*max(Wmsig),'g')
  xlabel('Time (hours)')
  ylabel('Wm motion (m)')
  title(commentbackslash(savename))
  drawnow
  
  %% Save mcl wavemaker control file and pdf:
  if (isave == 1)
    orient landscape
    printname = ['FlapMotion_' charid];
    print('-dpdf',['pdf/' printname])
    
    Data      = zeros(Nt,6);
    Data(:,1) = Wmsig*1000;
    Data(:,2) = Trigsig;
    write2mcl(['mcl/' savename],Data);
      
  end
end

