function [T,k,Cg,Cg_deep]=linwave2(given,value,given2,value2,R,h,ishow)

%function [T,k,Cg,Cg_deep]=linwave(given,value,given2,value2,R,h,ishow)
% Matlab function LINWAVE
% Purpose:
%      Outputs period, wavelength etc
%      according to the  
%      linear dispersion relation
%      'omega^2=gk tanh kh' when the quantities below
%      are given.
%
% Variables:
%      given   :  char, indicates what quantity 
%                       is given. One of the following:   
%                       'l' (lambda) 
%                       'k' (wave number)
%                       'T' (period)
%                       'o' (omega)
%      value   :  numerical value of quantity given
%
%      given2  :  char, either of the following:
%                       'A' (amplitude)
%                       's' (wave steepness s=kA=pi*H/lambda)
%                       'S' (wave steepness S=H/lambda)
%      value2  :  numerical value of quantity given2
% 
%      R       :  characteristic length (optional)
%      h       :  water depth (optional)
%      ishow   : if 0, do not display results

%
% Sept 2004, Trygve Kristiansen, Marintek
% 24.09.2014, Trygve Kristiansen, Marintek, "linwave2.m"
%

% Input parameters:
if (nargin<2)|(nargin==3)
  error(sprintf('%s','Wrong number of input arguments: ',num2str(nargin)));
elseif (nargin==2)
  A=1;
  R=1;
  eval([given '=' num2str(value) ';'])
  given2='A';
  inpstr=['Input: ',given,' = ',num2str(value)];
elseif (nargin==4)
  R=1;
  eval([given '=' num2str(value) ';'])
  eval([given2 '=' num2str(value2) ';'])
  inpstr=['Input: ',given,' = ',num2str(value) ', ' given2 ' = ',num2str(value2)];
elseif (nargin==5)
  eval([given '=' num2str(value) ';'])
  eval([given2 '=' num2str(value2) ';'])
  inpstr=['Input: ',given,' = ',num2str(value) ', ' given2 ' = ',num2str(value2) ', R = ',num2str(R)];
elseif (nargin==6 | nargin == 7)
  eval([given '=' num2str(value) ';'])
  eval([given2 '=' num2str(value2) ';'])
  inpstr=['Input: ',given,' = ',num2str(value) ', ' given2 ' = ',num2str(value2) ', R = ',num2str(R) ', h = ',num2str(h)];
end

% Constants:
g=9.81;

% Calculate quantities:
switch given
 case {'l'}                 % l==lambda
  k=2*pi/l;
  o=sqrt(g*k*tanh(k*h));
  T=2*pi/o;
 case {'k'}                 % k==wave number
  l=2*pi/k;
  o=sqrt(g*k*tanh(k*h));
  T=2*pi/o;
 case {'T'}                 % T==period
  o=2*pi/T;
  k=findk(T,h);
  l=2*pi/k;
 case {'o'}                 % o==omega
  T=2*pi/o;
  k=findk(T,h);
  l=2*pi/k;
 otherwise
end
switch given2
 case 'A'
  s=k*A;
  S=k*A/pi;
 case 's'
  A=s/k;
  S=pi*s;
 case 'S'
  A=pi*S/k;
  s=pi*S;
end
r=k*R;
kh=k*h;

% Phase and group velocity:
C=o/k;
Cg=C/2*(1+2*kh/sinh(2*kh));
Cg_deep=(0.5*sqrt(g/k));

% Linear wave maker:
HS1=4*(sinh(k*h)/(k*h))*(kh*sinh(kh)-cosh(kh)+1)/(sinh(2*kh)+2*kh);
HS1=4*(sinh(k*h)/(k*h))*(k*h*sinh(k*h)-cosh(k*h)+1)/(sinh(2*k*h)+2*k*h);
HS2=2*(cosh(2*kh)-1)/(sinh(2*kh)+2*kh);

if (ishow > 0)
  disp('---------------------------------------')
  disp(inpstr)
  disp(' ')
  disp(['  lambda         [m] = ',num2str(l)])
  disp(['  k            [1/m] = ',num2str(k)])
  disp(['  T              [s] = ',num2str(T)])
  disp(['  omega      [rad/s] = ',num2str(o)])
  disp(['  omega*A      [m/s] = ',num2str(o*A)])
  disp(['  omega^2*A  [m/s^2] = ',num2str(o^2*A)])
  disp(['  A              [m] = ',num2str(A)])
  disp(['  H/lambda       [1] = ',num2str(S)])
  disp(['  kA             [1] = ',num2str(s)])
  disp(['  C            [m/s] = ',num2str(C)])
  disp(['  Cg           [m/s] = ',num2str(Cg)])
  disp(['  Cg/C           [1] = ',num2str(Cg/C)])
  disp(['  Cg/Cg_deep     [1] = ',num2str(Cg/Cg_deep)])
  disp(['  kR             [1] = ',num2str(r)   '   (R=' num2str(R) ')'])
  disp(['  lambda/R       [1] = ',num2str(l/R) '   (R=' num2str(R) ')'])
  disp(['  kh             [1] = ',num2str(k*h)])
  disp(['  h/lambda       [1] = ',num2str(h/l)])
  disp(['  A/h            [1] = ',num2str(A/h)])
  disp(['  kA/tanhkh      [1] = ',num2str(k*A/tanh(k*h))])
  disp(['  H/S (flap)     [1] = ',num2str(HS1)])
  disp(['  H/S (piston)   [1] = ',num2str(HS2)])
  disp('---------------------------------------')
end
   





